@@ -1,5 +1,5 @@
 
-# $Id: BuyItNow.pm,v 1.7 2007/05/20 13:32:56 Daddy Exp $
+# $Id: BuyItNow.pm,v 1.8 2008/02/24 21:16:21 Daddy Exp $
 
 =head1 NAME
 
@@ -52,9 +52,12 @@ Martin Thurn C<mthurn@cpan.org>, L<http://tinyurl.com/nn67z>.
 package WWW::Search::Ebay::BuyItNow;
 
 use strict;
+use warnings;
+
+use WWW::Search::Ebay;
 use base 'WWW::Search::Ebay';
 our
-$VERSION = do { my @r = (q$Revision: 1.7 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 1.8 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 
 sub native_setup_search
   {
